# Rungeova funkcija je definisana sledecim izrazom
# f(x) = 1/(1 + 25*x^2)
#
# 1. Nacrtati funkciju na intervalu [-5,5].
# 2. Napraviti podelu intervala na 11 ekvidistantnih tacaka. 
# 3. Izvrsiti linearnu interpolaciju funkcije na podali iz koraka 2 i prikazati rezultat.
# 4. Izvrsiti interpolaciju kubnim splajnovima na podali iz koraka 2 i prikazati rezultat.
# 5. Razviti funkciju u Tejlorov red 5. stepena u tacki 0.
import matplotlib.pyplot as plt
import numpy as np
import scipy.interpolate as scp
#definisemo fukciju
def f(x):
    return 1/(1 + 25*x**2)

# kreiramo podelu iz koraka 2
xx = np.linspace(-5, 5, 11)
yy = f(xx)

# racunamo 1d interpolaciju
f1d = scp.interp1d(xx, yy)
# racunamo spline interpolaciju
fs = scp.CubicSpline(xx, yy)
# racunamo tejlorov razvoj u tacki 0
ft = scp.approximate_taylor_polynomial(f, 0, 5, 1, order=6)

# kreiramo novu podelu za prikaz rezultata
x = np.linspace(-5,5,200)
y = f(x)        # rungeova funkcija
y1d = f1d(x)    # 1d interpolacija
ys = fs(x)      # splajn interpolacija
yt = ft(x)      # tejlorov razvoj u tacki 0

plt.figure(1)
plt.plot(x, y, label=r"$f(x) = \frac{1}{1 + 25 \cdot x^2}$")
plt.plot(x, y1d, label=r"interp1d")
plt.plot(x, ys, label=r"spline interp")
plt.plot(x, yt, label=r"Tejlorov razvoj")

plt.title(r"Rungeova funkcija: $f(x) = \frac{1}{1 + 25 \cdot x^2}$")
plt.xlabel("x")
plt.ylabel("y")

plt.ylim([-0.2,1.2])

plt.legend()
plt.grid()
plt.tight_layout()
plt.show()