# Dat je skup podataka "iris.csv" sa informacijama
# o cvetovima irisa
#
# 1. Prikazati zavisnost SepalWidth od SepalLength i obojiti tacke u zavisnosti od vrste cveta.
# 2. Prikazati zavisnost PetalWidth od PetalLength i obojiti tacke u zavisnosti od vrste cveta.
# 3. Nacrtati histograme PetalLength po vrstama i obojiti ih u zavisnosti od vrste
# 4. Nacrtati histograme PetalWidth po vrstama i obojiti ih u zavisnosti od vrste
# 5. Nacrtati "pairplot" za date podatke. 
#    (pairplot je scatter matrica oblika svaki sa svakim za i!=j ili histogram za i== j)
import matplotlib.pyplot as plt
import pandas as pd

# ucitavamo skup podataka
data = pd.read_csv("iris.csv")
# stampamo kratke informacije o podacima
print(data.info())

# kreiramo filtere po klasama radi lakseg bojenja crteza 
setosa = data["Class"] == "Iris-setosa"
versicolor = data["Class"] == "Iris-versicolor"
virginica = data["Class"] == "Iris-virginica"

# zadatak 1
plt.figure(1)
plt.scatter(data.loc[setosa,"SepalLength"], data.loc[setosa, "SepalWidth"], label="Setosa")
plt.scatter(data.loc[versicolor,"SepalLength"], data.loc[versicolor, "SepalWidth"], label="Versicolor")
plt.scatter(data.loc[virginica,"SepalLength"], data.loc[virginica, "SepalWidth"], label="Virginica")

plt.title("Sepal length vs Sepal width")
plt.xlabel("Sepal length")
plt.ylabel("Sepal width")
plt.legend()
plt.grid()
plt.tight_layout()

# zadatak 2
plt.figure(2)
plt.scatter(data.loc[setosa,"PetalLength"], data.loc[setosa, "PetalWidth"], label="Setosa")
plt.scatter(data.loc[versicolor,"PetalLength"], data.loc[versicolor, "PetalWidth"], label="Versicolor")
plt.scatter(data.loc[virginica,"PetalLength"], data.loc[virginica, "PetalWidth"], label="Virginica")

plt.title("Petal length vs Petal width")
plt.xlabel("Petal length")
plt.ylabel("Petal width")
plt.legend()
plt.grid()
plt.tight_layout()

# zadatak 3
plt.figure(3)
plt.hist(data.loc[setosa,"PetalLength"], bins=10, label="Setosa")
plt.hist(data.loc[versicolor,"PetalLength"], bins=10, label="Versicolor")
plt.hist(data.loc[virginica,"PetalLength"], bins=10, label="Virginica")

plt.title("Petal length histogram")
plt.xlabel("Petal length")
plt.ylabel("Frequency")
plt.legend()
plt.grid()
plt.tight_layout()

# zadatak 4
plt.figure(4)
plt.hist(data.loc[setosa,"PetalWidth"], bins=10, label="Setosa")
plt.hist(data.loc[versicolor,"PetalWidth"], bins=10, label="Versicolor")
plt.hist(data.loc[virginica,"PetalWidth"], bins=10, label="Virginica")

plt.title("Petal width histogram")
plt.xlabel("Petal width")
plt.ylabel("Frequency")
plt.legend()
plt.grid()
plt.tight_layout()


# crtamo pairplot
fig, axis = plt.subplots(nrows=5, ncols=5, figsize=(10, 6), layout='constrained')
# u petlji crtamo jedan po jedan crtez
for i in range(0,5):
    for j in range(0,5):
        plot_single(axis, i, j, )


plt.show()

def plot_single_pair(ax, feature_ind1, feature_ind2, _X, _y, _features, colormap):
    """Plots single pair of features.

    Parameters
    ----------
    ax : Axes
        matplotlib axis to be plotted
    feature_ind1 : int
        index of first feature to be plotted
    feature_ind2 : int
        index of second feature to be plotted
    _X : numpy.ndarray
        Feature dataset of of shape m x n
    _y : numpy.ndarray
        Target list of shape 1 x n
    _features : list of str
        List of n feature titles
    colormap : dict
        Color map of classes existing in target

    Returns
    -------
    None
    """

    # Plot distribution histogram if the features are the same (diagonal of the pair-plot).
    if feature_ind1 == feature_ind2:
        tdf = pd.DataFrame(_X[:, [feature_ind1]], columns = [_features[feature_ind1]])
        tdf['target'] = _y
        for c in colormap.keys():
            tdf_filtered = tdf.loc[tdf['target']==c]
            ax[feature_ind1, feature_ind2].hist(tdf_filtered[_features[feature_ind1]], color = colormap[c], bins = 30)
    else:
        # other wise plot the pair-wise scatter plot
        tdf = pd.DataFrame(_X[:, [feature_ind1, feature_ind2]], columns = [_features[feature_ind1], _features[feature_ind2]])
        tdf['target'] = _y
        for c in colormap.keys():
            tdf_filtered = tdf.loc[tdf['target']==c]
            ax[feature_ind1, feature_ind2].scatter(x = tdf_filtered[_features[feature_ind2]], y = tdf_filtered[_features[feature_ind1]], color=colormap[c])

    # Print the feature labels only on the left side of the pair-plot figure
    # and bottom side of the pair-plot figure. 
    # Here avoiding printing the labels for inner axis plots.
    if feature_ind1 == len(_features) - 1:
        ax[feature_ind1, feature_ind2].set(xlabel=_features[feature_ind2], ylabel='')
    if feature_ind2 == 0:
        if feature_ind1 == len(_features) - 1:
            ax[feature_ind1, feature_ind2].set(xlabel=_features[feature_ind2], ylabel=_features[feature_ind1])
        else:
            ax[feature_ind1, feature_ind2].set(xlabel='', ylabel=_features[feature_ind1])

def myplotGrid(X, y, features, colormap={0: "red", 1: "green", 2: "blue"}):
    """Plots a pair grid of the given features.

    Parameters
    ----------
    X : numpy.ndarray
        Dataset of shape m x n
    y : numpy.ndarray
        Target list of shape 1 x n
    features : list of str
        List of n feature titles

    Returns
    -------
    None
    """

    feature_count = len(features)
    # Create a matplot subplot area with the size of [feature count x feature count]
    fig, axis = plt.subplots(nrows=feature_count, ncols=feature_count)
    # Setting figure size helps to optimize the figure size according to the feature count.
    fig.set_size_inches(feature_count * 4, feature_count * 4)

    # Iterate through features to plot pairwise.
    for i in range(0, feature_count):
        for j in range(0, feature_count):
            plot_single_pair(axis, i, j, X, y, features, colormap)

    plt.show()